SystemOrganization addCategory: #'SqSync-Shared'!
SystemOrganization addCategory: #'SqSync-Client'!
SystemOrganization addCategory: #'SqSync-Server'!
SystemOrganization addCategory: #'SqSync-Tests'!

OrderedCollection subclass: #SySharedList
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SySharedList commentStamp: 'mt 5/9/2010 17:04:27.753' prior: 0!
This is meant to be a thread-safe version of an OrderedCollection. See the "list" protocol to explore selectors which are thread-safe.!

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:52:24.122'!
add: anElement

	^ self mutex critical: [super add: anElement].! !

!SySharedList methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:40:52.25'!
mutex

	^ mutex ifNil: [mutex := Mutex new]! !

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:53:19.735'!
remove: anElement ifAbsent: aBlock
	"#remove: should also be guarded now."
	
	^ self mutex critical: [super remove: anElement ifAbsent: aBlock].! !

Object subclass: #SyNetworkDataProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyNetworkDataProcessor commentStamp: 'mt 5/9/2010 17:01:16.718' prior: 0!
This is a place holder for the "P"" in our diagram.

Current contract: NO STATE - ONLY BEHAVIOR!

!SyNetworkDataProcessor class methodsFor: 'processing' stamp: 'mt 5/9/2010 17:02:23.913'!
processDataFrom: aStream
	"Hint: Expect stream API from SocketStream. Other Streams should work as well."! !

Object subclass: #SyTcpServer
	instanceVariableNames: 'listenSocket listenProcess connections connectionProcesses connectionChangesMonitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Server'!

!SyTcpServer methodsFor: 'connection open/close' stamp: 'rkrk 5/14/2010 17:50'!
close

	"Closing listen part"
	listenProcess terminate.
	self listenSocket close.
	listenSocket := nil.
	
	"Closing connection part"
	self connectionProcesses do: [:proc | proc terminate].
	self connections do: [:ea | ea close].! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:25:52.291'!
connectionChangesMonitor

	^ connectionChangesMonitor! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'SR 5/10/2010 17:28:42.275'!
connectionProcesses

	^ connectionProcesses ifNil: [connectionProcesses := SySharedList new]! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:46:22.708'!
connections

	^ connections ifNil: [connections := SySharedList new]! !

!SyTcpServer methodsFor: 'synchronizing' stamp: 'rkrk 5/14/2010 18:20'!
connectionsChanged

	[self connectionChangesMonitor signalAll]
		on: Error do: [:e |
			self
				log: 'Problem signaling connectionChangesMonitor';
				log: e.
			self halt.].! !

!SyTcpServer methodsFor: 'waiting' stamp: 'SR 5/10/2010 18:26:19.094'!
initialize

	super initialize.
	
	connectionChangesMonitor := Monitor new.! !

!SyTcpServer methodsFor: 'testing' stamp: 'mt 5/9/2010 16:17:23.888'!
isListening

	^ self listenSocket isWaitingForConnection! !

!SyTcpServer methodsFor: 'connection open/close' stamp: 'rkrk 5/14/2010 13:30'!
listenOn: aPort

	self isListening ifTrue: [self error: 'Server already listening!!'].
	self listenSocket listenOn: aPort backlogSize: 4.
	self startListenThread.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 15:58:25.953'!
listenSocket

	^ listenSocket ifNil: [listenSocket := Socket newTCP]! !

!SyTcpServer methodsFor: 'debugging' stamp: 'rkrk 5/14/2010 17:31'!
log: msg

	Transcript cr; show: msg asString.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'rkrk 5/14/2010 13:47'!
newConnectionOn: socket

	| con |
	con := SyTcpSocket on: socket.
	self connections add: con.
	self startConnectionThreadFor: con.
! !

!SyTcpServer methodsFor: 'accessing' stamp: 'mt 5/9/2010 17:12:59.484'!
port

	^ self listenSocket port! !

!SyTcpServer methodsFor: 'threads' stamp: 'rkrk 5/14/2010 17:32'!
startConnectionThreadFor: aTcpSocket

	| connectionProcess |
	connectionProcess := [
		self connectionChangesMonitor critical: [
			[aTcpSocket isConnected] whileTrue: [
				aTcpSocket waitForDataIfClosed: [
					self connections remove: aTcpSocket.
					self log: 'Some client socket closed'; log: aTcpSocket.
					self connectionsChanged].
			SyNetworkDataProcessor processDataFrom: aTcpSocket]]].
	self connectionProcesses add: (connectionProcess forkAt: 35).! !

!SyTcpServer methodsFor: 'threads' stamp: 'rkrk 5/14/2010 17:32'!
startListenThread
	
	listenProcess := [
		[self isListening] whileTrue: [
			self connectionChangesMonitor critical: [
				(self listenSocket waitForAcceptFor: 10) ifNotNilDo: [:socket |
					self newConnectionOn: socket.
					self log: 'New client connected'; log: socket.
					self connectionsChanged]]]] forkAt: 35.! !

!SyTcpServer methodsFor: 'waiting' stamp: 'mt 5/10/2010 11:27:21.847'!
waitForNewConnectionFrom: nameOrAddressString port: aPort

	| remoteAddress |
	remoteAddress := NetNameResolver addressForName: nameOrAddressString timeout: 20.
	
	[
		self connections includes: [:tcpSocket | 
			tcpSocket remoteAddress = remoteAddress
			and: [tcpSocket remotePort = aPort]].
		Processor yield
	] whileFalse.! !

TestCase subclass: #SyCommunicationModuleTest
	instanceVariableNames: 'tcpServer tcpSocket openClientSockets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SyCommunicationModuleTest class methodsFor: 'helper' stamp: 'rkrk 5/14/2010 17:02'!
closeAllSockets

	Socket registry keys
		select: [:ea | ea isWaitingForConnection or: [ea isOtherEndClosed] or: [ea isConnected]]
		thenDo: [:ea | ea close].! !

!SyCommunicationModuleTest class methodsFor: 'constants' stamp: 'rkrk 5/14/2010 14:24'!
defaultPort

	^ 22701! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 18:19'!
connectTCPClientToServer

	tcpSocket := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.
	openClientSockets add: tcpSocket.
	^ tcpSocket.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 18:15'!
disconnectTCPClientsFromServer

	tcpSocket ifNotNil: [tcpSocket close. tcpSocket := nil].
	openClientSockets do: [:ea | ea close.]! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 17:15'!
maybeWaitForServerThenDo: block

	"sometimes the server is already done before waiting is started so try the block
	first then optionally wait"
	tcpServer connectionChangesMonitor critical: [
		block on: Exception do: [
			Transcript cr;
				show: 'maybeWaitForServerThenDo: first try was failure in';
					show: self selector.
			tcpServer connectionChangesMonitor wait.
			block value]].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'rkrk 5/14/2010 18:17'!
setUp

	super setUp.
	openClientSockets := OrderedCollection new.
	Transcript cr; show: 'start '; show: self selector.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 11:39:41.976'!
startTcpServer

	tcpServer ifNil: [tcpServer := SyTcpServer new].
	tcpServer isListening
		ifFalse: [tcpServer listenOn: self class defaultPort].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'SR 5/10/2010 17:17:02.712'!
stopTcpServer

	(tcpServer notNil and: [tcpServer isListening])
		ifTrue: [tcpServer close].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'rkrk 5/14/2010 18:15'!
tearDown

	super tearDown.
	Transcript cr; show: 'end '; show: self selector.
	[self
		disconnectTCPClientsFromServer;
		stopTcpServer
	] on: Error do: [:e | Transcript cr; show: 'Error in tearDown'; print: e].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 14:23'!
test01TcpServerStartup

	| sut |
	sut := SyTcpServer new.
	
	self deny: sut isListening.
	
	[sut listenOn: self class defaultPort.
	self
		assert: sut isListening;
		assert: self class defaultPort equals: sut port;
		assert: sut connections isEmpty.
	] ensure: [sut close].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 14:24'!
test02TcpServerShutdown
	
	self
		startTcpServer;
		stopTcpServer;
		deny: tcpServer isListening;
		assert: tcpServer connections isEmpty;
		assert: (Socket registry keys noneSatisfy: [:socket | socket isWaitingForConnection]).! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 16:56'!
test03FailingTcpSocketConnection

	"try to connect to a server. This should either throw a ConnectionTimedOut error.
	Since we currently cannot control the timeout period of sockets and the default of 
	[Socket standardTimeout] = 45 is used we end the test earlier and just assume
	that the connection can't be established."
	<timeout: 6> "1s more than needed"
	| connectionFailed connectReturned connectProcess |
	connectionFailed := false.
	connectReturned := false.
	connectProcess := [
		[self connectTCPClientToServer. connectReturned := true]
			on: ConnectionTimedOut
			do: [connectionFailed := true]] fork.
	self wait: 5. "seconds"
	connectProcess terminate.
	connectionFailed ifTrue: [^self].
	connectReturned ifTrue: [^self]. "assume it works"
	self
		assert: false
		description: '#connectTCPClientToServer was successful although no server was started!!'.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 17:13'!
test04TcpSocketConnection

	<timeout: 5>
	self
		startTcpServer;
		connectTCPClientToServer;
		assert: tcpSocket isConnected;
		maybeWaitForServerThenDo: [self assert: 1 equals: tcpServer connectionProcesses size].! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'rkrk 5/14/2010 18:19'!
test05TcpServerIncomingConnections

	self
		startTcpServer;
		assert: tcpServer connections isEmpty;
		connectTCPClientToServer;
		maybeWaitForServerThenDo: [self assert: tcpServer connections size = 1].

	self connectTCPClientToServer.
		
	self
		maybeWaitForServerThenDo: [self assert: tcpServer connections size = 2];
		disconnectTCPClientsFromServer;
		assert: tcpServer connections size = 1.
	
	self assert: tcpServer connections isEmpty.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'rkrk 5/14/2010 16:55'!
wait: seconds

	(Delay forSeconds: seconds) wait.! !

SocketStream subclass: #SyTcpSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyTcpSocket class methodsFor: 'instance creation' stamp: 'mt 5/10/2010 11:05:32.461'!
connectToHost: nameOrAddressString port: anInteger
	
	^ self
		openConnectionToHostNamed: nameOrAddressString
		port: anInteger! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:48.023'!
remoteAddress

	^ self socket remoteAddress! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:53.871'!
remotePort

	^ self socket remotePort! !

!SyTcpSocket methodsFor: 'waiting' stamp: 'mt 5/9/2010 17:23:15.848'!
waitForDataIfClosed: aBlock

	^ self socket waitForDataIfClosed: aBlock! !
